/*
**   _______              __
**  / ___/ /  ___  __ _  / /  ___
** / /__/ _ \/ _ \/  ' \/ _ \/ _ \
** \___/_//_/\___/_/_/_/_.__/\___/ 
**
** This software is copyright (C) by the Lawrence Berkeley
** National Laboratory.  Permission is granted to reproduce
** this software for non-commercial purposes provided that
** this notice is left intact.
** 
** It is acknowledged that the U.S. Government has rights to
** this software under Contract DE-AC03-765F00098 between
** the U.S. Department of Energy and the University of
** California.
**
** This software is provided as a professional and academic
** contribution for joint exchange.  Thus it is experimental,
** is provided ``as is'', with no warranties of any kind
** whatsoever, no support, no promise of updates, or printed
** documentation.  By using this software, you acknowledge
** that the Lawrence Berkeley National Laboratory and
** Regents of the University of California shall have no
** liability with respect to the infringement of other
** copyrights by any part of this software.
**
*/
// Author: Ted Sternberg

#include "Intvect.h"

/** No version for Realvect.  If you ever need it, need to define tolerance.
*/
template<> bool
Triple<int>::operator==( Triple<int> const & that ) const
{
    if( (m_data[0] == that.m_data[0])
    &&  (m_data[1] == that.m_data[1])
    &&  (m_data[2] == that.m_data[2]) )
    {
        return true;
    } else
    {
        return false;
    }
}

/** Ditto, no version for Realvect. */
template<> bool
Triple<int>::operator!=( Triple<int> const & that ) const
{
    return ! operator==(that);
}

/** Only needed for Intvect, and always rounds down. */
template<> void
Triple<int>::MultiplyAndRoundDown( double x )
{
    int temp;
    for( int i=0;i<THREE;++i )
    {
        temp = int(floor(m_data[i] * x + 1E-10));
        // We want to round down, but we need the 1E-10 to prevent 5.0,
        // represented internally as 4.99999999, from being "rounded" all the
        // way down to 4.
        m_data[i] = temp;
    }
}


/** Imax, Imin, Jmax, Jmin: return one of two Intvect args, according to which
 *  one of them has the larger i or j coordinate.
*/
/*static*/ Intvect
Intvect::Imax( Intvect const & iv1, Intvect const & iv2 )
{
    return (iv1.i() > iv2.i()) ? iv1:iv2;
}
/*static*/ Intvect
Intvect::Imin( Intvect const & iv1, Intvect const & iv2 )
{
    return (iv1.i() > iv2.i()) ? iv2:iv1;
}
/*static*/ Intvect
Intvect::Jmax( Intvect const & iv1, Intvect const & iv2 )
{
    return (iv1.j() > iv2.j()) ? iv1:iv2;
}
/*static*/ Intvect
Intvect::Jmin( Intvect const & iv1, Intvect const & iv2 )
{
    return (iv1.j() > iv2.j()) ? iv2:iv1;
}


template<> Triple<int>
MultiplyAndRoundDown( Triple<int> const & iv1, double x )
{
    Triple<int> result( iv1 );
    result.MultiplyAndRoundDown( x );
    return result;
}

#if __GNUC_MINOR__ > 3
#define templateangleangle template<>
#else
#define templateangleangle
#endif
templateangleangle hid_t Triple<int>::s_HDF5Datatype(0);
templateangleangle hid_t Triple<float>::s_HDF5Datatype(0);
templateangleangle hid_t Triple<double>::s_HDF5Datatype(0);
#undef templateangleangle
